import sys
from asyncio import subprocess

from aiohttp import ClientSession, ClientTimeout
from nonebot import on_command
from nonebot.adapters.onebot.v11 import Message
from nonebot.matcher import Matcher
from nonebot.params import CommandArg
from packaging import version

from amrita.cli import IS_IN_VENV
from amrita.plugins.perm.API.admin import is_lp_admin
from amrita.utils.utils import get_amrita_version

amrita = on_command(
    "miniagent",
    aliases={"MiniAgent", "amrita", "Amrita"},
    priority=10,
    block=True,
    permission=is_lp_admin,
)


@amrita.handle()
async def _(matcher: Matcher, args: Message = CommandArg()):
    arg_list = args.extract_plain_text().strip().split()
    match len(arg_list):
        case 1:
            if arg_list[0] == "version":
                await matcher.finish(
                    f"MiniAgent v{get_amrita_version()} (based on Amrita)"
                )

            if arg_list[0] == "update":
                await matcher.send("正在检查 MiniAgent 更新...")

                try:
                    timeout = ClientTimeout(total=10)
                    async with ClientSession(timeout=timeout) as session:
                        metadata = None
                        package_name = None
                        for candidate in (
                            "minichatagent",
                            "miniagent",
                            "amrita",
                        ):
                            async with session.get(
                                f"https://pypi.org/pypi/{candidate}/json"
                            ) as response:
                                if response.status != 200:
                                    continue
                                metadata = await response.json()
                                package_name = candidate
                                break

                    if not package_name or not metadata or metadata.get("releases") == {}:
                        await matcher.send("检查更新失败，请稍后再试")
                        return

                    latest_version = max(metadata["releases"].keys(), key=version.parse)

                except Exception:
                    await matcher.finish("错误：无法检查更新")
                else:
                    if version.parse(latest_version) > version.parse(get_amrita_version()):
                        await matcher.send(
                            f"新版本已就绪: {latest_version}，正在更新..."
                        )
                        try:
                            install_cmd = (
                                f"uv add {package_name}=={latest_version}"
                                if IS_IN_VENV
                                else (
                                    f"pip install {package_name}=={latest_version}"
                                    + (
                                        " --break-system-packages"
                                        if sys.platform.lower() == "linux"
                                        else ""
                                    )
                                )
                            )
                            await (
                                await subprocess.create_subprocess_shell(install_cmd)
                            ).wait()
                            await matcher.send("完成更新，请重新启动程序以应用更改。")
                        except Exception as e:
                            await matcher.send("更新失败：" + str(e))
                    else:
                        await matcher.send("已是最新版本。")
                return

            await matcher.finish("错误：参数错误")

        case _:
            await matcher.finish(
                "错误：需要1个参数！\n输入格式：/miniagent [参数]\n可用：version|update\n别名：/amrita"
            )



from importlib import metadata

__version__ = "unknown"

for _dist_name in ("minichatagent", "miniagent", "amrita"):
    try:
        __version__ = metadata.version(_dist_name)
        break
    except metadata.PackageNotFoundError:
        continue


def get_amrita_version():
    return __version__



from nonebot import get_driver
from nonebot.adapters.onebot.v11 import Event

from ..config import DataManager
from .rules import UserPermissionChecker

ENV_ADMINS = get_driver().config.superusers


async def is_lp_admin(event: Event) -> bool:
    """
    判断是否为管理员
    """
    user_id = event.get_user_id()

    return user_id in ENV_ADMINS or (
        await (UserPermissionChecker("lp.admin").checker())(event)
        and DataManager().config.enable
    )



from pathlib import Path
from typing import Any

from nonebot import logger

from amrita.plugins.chat.config import config_manager
from amrita.plugins.chat.utils.models import InsightsModel
from amrita.plugins.webui.API import (
    JSONResponse,
    PageContext,
    PageResponse,
    Request,
    SideBarCategory,
    SideBarManager,
    TemplatesManager,
    on_page,
)

# 导入API路由
from amrita.plugins.webui.API import app as router

TemplatesManager().add_templates_dir(Path(__file__).resolve().parent / "templates")

SideBarManager().add_sidebar_category(
    SideBarCategory(name="聊天管理", icon="fa fa-comments", url="#")
)

KEY_PLACEHOLDER = "••••••••"


@router.post("/api/chat/models")
async def create_model(request: Request):
    try:
        data: dict[str, Any] = await request.json()
        name = data.get("name")
        model = data.get("model", "")
        base_url = data.get("base_url", "")
        api_key = data.get("api_key", "")
        protocol = data.get("protocol", "__main__")
        multimodal = data.get("multimodal", False)
        thought_chain_model = data.get("thought_chain_model", False)

        if not name:
            return JSONResponse(
                {"success": False, "message": "缺少模型预设名称"}, status_code=400
            )

        # Input validation and sanitization
        if not isinstance(name, str) or len(name) > 100:
            return JSONResponse(
                {"success": False, "message": "无效的模型预设名称"}, status_code=400
            )

        if not isinstance(model, str) or len(model) > 200:
            return JSONResponse(
                {"success": False, "message": "无效的模型名称"}, status_code=400
            )

        if base_url and (not isinstance(base_url, str) or len(base_url) > 500):
            return JSONResponse(
                {"success": False, "message": "无效的API地址"}, status_code=400
            )

        if api_key and (not isinstance(api_key, str) or len(api_key) > 200):
            return JSONResponse(
                {"success": False, "message": "无效的API密钥"}, status_code=400
            )

        if protocol and (not isinstance(protocol, str) or len(protocol) > 50):
            return JSONResponse(
                {"success": False, "message": "无效的协议"}, status_code=400
            )

        # Sanitize inputs to prevent injection
        name = name.strip()
        model = model.strip()
        base_url = base_url.strip() if base_url else ""
        api_key = api_key.strip() if api_key else ""
        protocol = protocol.strip() if protocol else "__main__"

        # 创建模型预设
        from amrita.plugins.chat.config import ModelPreset

        preset = ModelPreset(
            name=name,
            model=model,
            base_url=base_url,
            api_key=api_key,
            protocol=protocol,
            multimodal=multimodal,
            thought_chain_model=thought_chain_model,
        )

        # 保存模型预设到文件
        preset_path = config_manager.custom_models_dir / f"{name}.json"
        preset.save(preset_path)

        # 重新加载模型列表
        await config_manager.get_all_presets(cache=False)

        return JSONResponse(
            {"success": True, "message": f"模型预设 {name} 创建成功"}, status_code=200
        )
    except Exception as e:
        logger.opt(exception=e, colors=True).error("创建模型预设失败")
        return JSONResponse(
            {"success": False, "message": f"创建模型预设失败: {e!s}"},
            status_code=500,
        )


@router.put("/api/chat/models/{name}")
async def update_model(request: Request, name: str):
    try:
        # 获取现有的模型预设
        preset = await config_manager.get_preset(name, fix=False, cache=False)

        if not preset:
            return JSONResponse(
                {"success": False, "message": f"模型预设 {name} 不存在"},
                status_code=404,
            )

        data: dict[str, Any] = await request.json()

        # 更新字段
        for key, value in data.items():
            if hasattr(preset, key):
                setattr(preset, key, value)

        # 保存模型预设到文件
        preset_path = config_manager._model_name2file[name]

        preset.save(preset_path)

        # 重新加载模型列表
        await config_manager.get_all_presets(cache=False)

        return JSONResponse(
            {"success": True, "message": f"模型预设 {name} 更新成功"}, status_code=200
        )
    except Exception as e:
        logger.opt(exception=e, colors=True).error("[webui] 模型预设更新失败: %s", e)
        return JSONResponse(
            {"success": False, "message": f"更新模型预设失败: {e!s}"},
            status_code=500,
        )


@router.delete("/api/chat/models/{name}")
async def delete_model(name: str):
    try:
        preset_path = config_manager._model_name2file[name]

        if not preset_path.exists():
            return JSONResponse(
                {"success": False, "message": f"模型预设 {name} 不存在"},
                status_code=404,
            )

        # 删除文件
        preset_path.unlink()

        # 重新加载模型列表
        await config_manager.get_all_presets(cache=False)
        del config_manager._model_name2file[name]

        return JSONResponse(
            {"success": True, "message": f"模型预设 {name} 删除成功"}, status_code=200
        )
    except Exception as e:
        logger.opt(exception=e, colors=True).error(f"Error in delete preset {name}")
        return JSONResponse(
            {"success": False, "message": f"删除模型预设失败: {e!s}"},
            status_code=500,
        )


@router.get("/api/chat/models")
async def get_models():
    try:
        models = await config_manager.get_all_presets(cache=False)
        model_data = [
            {
                "name": model.name,
                "model": model.model,
                "base_url": model.base_url,
                "api_key": KEY_PLACEHOLDER,
                "protocol": model.protocol,
                "multimodal": model.multimodal,
                "thought_chain_model": model.thought_chain_model,
            }
            for model in models
        ]

        return JSONResponse({"success": True, "models": model_data}, status_code=200)
    except Exception as e:
        logger.opt(exception=e, colors=True).error("获取模型预设列表失败")
        return JSONResponse(
            {"success": False, "message": f"获取模型预设列表失败: {e!s}"},
            status_code=500,
        )


@on_page("/manage/chat/function", page_name="信息统计", category="聊天管理")
async def _(ctx: PageContext):
    insight = await InsightsModel.get()
    insight_all = await InsightsModel.get_all()
    return PageResponse(
        name="function.html",
        context={
            "token_prompt": insight.token_input,
            "token_completion": insight.token_output,
            "usage_count": insight.usage_count,
            "chart_data": [
                {
                    "date": i.date,
                    "token_input": i.token_input,
                    "token_output": i.token_output,
                    "usage_count": i.usage_count,
                }
                for i in insight_all
            ],
        },
    )


@on_page("/manage/chat/models", page_name="模型预设", category="聊天管理")
async def _(ctx: PageContext):
    models = await config_manager.get_all_presets(cache=False)
    current_default = config_manager.config.preset

    model_data = [
        {
            "name": model.name,
            "model": model.model,
            "base_url": model.base_url,
            "api_key": KEY_PLACEHOLDER,
            "protocol": model.protocol,
            "multimodal": model.multimodal,
            "thought_chain_model": model.thought_chain_model,
        }
        for model in models
    ]

    return PageResponse(
        name="models.html",
        context={
            "models": model_data,
            "current_default": current_default,
            "key_placeholder": KEY_PLACEHOLDER,
        },
    )



import secrets
import string

from nonebot import get_plugin_config
from pydantic import BaseModel


def generate_secure_password(length: int = 16) -> str:
    """
    Generate a secure random password
    """
    characters = string.ascii_letters + string.digits + "!@#$%^&*()"
    while True:
        password = "".join(secrets.choice(characters) for _ in range(length))
        # Ensure password contains at least one uppercase, one lowercase, one digit, and one special character
        if (any(c.isupper() for c in password) and
            any(c.islower() for c in password) and
            any(c.isdigit() for c in password) and
            any(c in "!@#$%^&*()" for c in password)):
            return password


class Config(BaseModel):
    """
    Configuration for webui
    """

    webui_enable: bool = True
    webui_user_name: str = "admin"
    webui_password: str = ""


def get_webui_config() -> Config:
    config = get_plugin_config(Config)

    # Generate secure password if default/empty password is detected
    if not config.webui_password or config.webui_password == "admin123":
        # This is a security improvement - generate a secure random password
        # In production, this should be handled during setup process
        secure_password = generate_secure_password()
        # Note: In a real implementation, this would be persisted to config
        # For now, we'll return the generated password but it won't persist
        config.webui_password = secure_password

    return config
